-- Deploy camper:campsite_feature to pg
-- requires: roles
-- requires: schema_camper
-- requires: campsite
-- requires: icon
-- requires: user_profile

begin;

set search_path to camper, public;

create table campsite_feature (
	campsite_feature_id integer generated by default as identity primary key,
	campsite_id integer not null references campsite,
	icon_name text not null references icon,
	name text not null constraint name_not_empty check(length(trim(name)) > 0),
	position integer not null default 2147483647
);

grant select on table campsite_feature to guest;
grant select on table campsite_feature to employee;
grant select, insert, update, delete on table campsite_feature to admin;

alter table campsite_feature enable row level security;

create policy guest_ok
on campsite_feature
for select
using (true)
;

create policy insert_to_company
on campsite_feature
for insert
with check (
	exists (select 1 from campsite join user_profile using (company_id) where campsite.campsite_id = campsite_feature.campsite_id)
)
;

create policy update_company
on campsite_feature
for update
using (
	exists (select 1 from campsite join user_profile using (company_id) where campsite.campsite_id = campsite_feature.campsite_id)
)
;

create policy delete_from_company
on campsite_feature
for delete
using (
	exists (select 1 from campsite join user_profile using (company_id) where campsite.campsite_id = campsite_feature.campsite_id)
)
;

commit;
