pragma ComponentBehavior: Bound
import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import Camper

Page {
    id: page

    property real dayWidth: 24
    property date fromDate: "2024-11-01"
    property date toDate: "2025-01-31"

    title: qsTr("Reservations")

    header: ToolBar {
        RowLayout {
            anchors.fill: parent

            ToolButton {
                action: logoutAction
            }
        }
    }

    ListView {
        id: lodgingList

        ScrollBar.vertical: verticalScroll
        anchors.bottom: parent.bottom
        anchors.left: parent.left
        anchors.top: parent.top
        headerPositioning: ListView.OverlayHeader
        model: timelineListModel
        width: 100

        delegate: Label {
            required property string name

            text: name
        }
        header: Pane {
            height: timelineList.headerItem.height
            z: 2

            Label {
                text: qsTr("Lodging")
            }
        }
    }

    ListView {
        id: timelineList

        anchors.bottom: parent.bottom
        anchors.left: lodgingList.right
        anchors.right: parent.right
        anchors.top: parent.top
        clip: true
        contentWidth: headerItem.implicitWidth
        flickableDirection: Flickable.AutoFlickDirection
        headerPositioning: ListView.OverlayHeader
        model: timelineListModel

        ScrollBar.horizontal: ScrollBar {
        }
        ScrollBar.vertical: ScrollBar {
            id: verticalScroll

        }
        delegate: TimelineView {
            required property TimelineModel timeline

            dayWidth: page.dayWidth
            fromDate: page.fromDate
            height: 16
            model: timeline
            toDate: page.toDate
            viewportWidth: ListView.view.width
            viewportX: ListView.view.contentX

            delegate: Rectangle {
                border.color: "black"
                border.width: 1
                color: "blue"
            }
        }
        header: Pane {
            leftPadding: 0
            rightPadding: 0
            z: 2

            Column {
                TimelineMonthRow {
                    dayWidth: page.dayWidth
                    fromDate: page.fromDate
                    toDate: page.toDate
                }

                TimelineDayRow {
                    dayWidth: page.dayWidth
                    fromDate: page.fromDate
                    toDate: page.toDate
                }
            }
        }
    }

    TimelineListModel {
        id: timelineListModel

    }

    MnemonicAction {
        id: logoutAction

        icon.name: "system-log-out"
        mnemonic: qsTr("Log &out")

        onTriggered: function () {
            Database.close();
        }
    }
}
