#include "timelinemodel.h"

TimelineModel::TimelineModel(QObject *parent)
    : QObject{parent}
    , m_items()
{}

TimelineModel::~TimelineModel()
{
    clear();
}

void TimelineModel::update(const QList<Item *> &items)
{
    clear();
    m_items.assign(items.begin(), items.end());
}

std::pair<qsizetype, qsizetype> TimelineModel::indexesOf(QDate from, QDate to) const
{
    qsizetype begin = searchIndex(from);
    if (begin == -1) {
        return std::make_pair(begin, begin);
    }

    while (begin > 0) {
        --begin;
        const Item *item = m_items.at(begin);
        if (m_items.at(begin)->departure() < from) {
            ++begin;
            break;
        }
    }

    for (qsizetype end = begin + 1; end < m_items.count(); end++) {
        if (m_items.at(end)->arrival >= to) {
            return std::make_pair(begin, end);
        }
    }
    return std::make_pair(begin, m_items.count());
}

const TimelineModel::Item *TimelineModel::at(qsizetype index) const
{
    return m_items.at(index);
}

void TimelineModel::clear()
{
    qDeleteAll(m_items);
    m_items.clear();
}

qsizetype TimelineModel::searchIndex(QDate date) const
{
    if (m_items.isEmpty()) {
        return -1;
    }

    if (m_items.last()->arrival < date) {
        return m_items.size();
    }

    qsizetype begin = 0;
    qsizetype end = m_items.size() - 1;
    qsizetype middle;
    for (middle = end / 2; begin <= end; middle = (begin + end) / 2) {
        if (m_items.at(middle)->arrival < date) {
            begin = middle + 1;
        } else {
            end = middle - 1;
        }
    }
    return middle + (m_items.at(middle)->arrival < date ? 1 : 0);
}

#include "moc_timelinemodel.cpp"
