#ifndef TIMELINEMONTHMODEL_H
#define TIMELINEMONTHMODEL_H

#include <QAbstractListModel>
#include <QList>
#include <QLocale>
#include <QtQmlIntegration>

class TimelineMonthModel : public QAbstractListModel
{
    Q_OBJECT
    QML_ELEMENT

    Q_PROPERTY(QDate fromDate READ fromDate WRITE setFromDate NOTIFY fromDateChanged)
    Q_PROPERTY(QDate toDate READ toDate WRITE setToDate NOTIFY toDateChanged)
    Q_PROPERTY(QLocale locale READ locale WRITE setLocale NOTIFY localeChanged)

public:
    enum {
        DayCount = Qt::UserRole,
    };

    explicit TimelineMonthModel(QObject *parent = nullptr);

    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
    QHash<int, QByteArray> roleNames() const override;

    QDate fromDate() const;
    void setFromDate(QDate date);

    QDate toDate() const;
    void setToDate(QDate date);

    QLocale locale() const;
    void setLocale(const QLocale &locale);

signals:
    void fromDateChanged(QDate date);
    void toDateChanged(QDate date);
    void localeChanged(const QLocale &locale);

private:
    Q_DISABLE_COPY_MOVE(TimelineMonthModel)

    struct Month
    {
        int number;
        int days;
    };

    void updateMonths();

    QDate m_fromDate;
    QDate m_toDate;
    QLocale m_locale;
    QList<Month> m_months;
};

#endif // TIMELINEMONTHMODEL_H
